@@ -3,7 +3,7 @@
    | PHP Version 5 / Imagick	                                          |
    +----------------------------------------------------------------------+
    | Copyright (c) 2006-2013 Mikko Koppanen, Scott MacVicar               |
-   | Imagemagick (c) ImageMagick Studio LLC                               |
+   | ImageMagick (c) ImageMagick Studio LLC                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -29,6 +29,8 @@
 #  include <wand/MagickWand.h>
 #endif
 
+#include "php_imagemagick_version_defs.h"
+
 #if MagickLibVersion >= 0x700
 #include "shim_im6_to_im7.h"
 #endif
@@ -42,6 +44,16 @@
 #include "php_ini.h"
 #include "Zend/zend.h"
 
+#if PHP_VERSION_ID >= 80000
+	#define HAVE_LOCALE_H
+	#define TSRMLS_C
+	#define TSRMLS_CC
+	#define TSRMLS_D
+	#define TSRMLS_DC
+	#define TSRMLS_FETCH()
+	#define TSRMLS_SET_CTX(ctx)
+#endif
+
 /* Include locale header */
 #ifdef HAVE_LOCALE_H
 # include <locale.h>
@@ -61,16 +73,21 @@
 #endif
 
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	#define im_long zend_long
 #else
 	#define im_long long
 #endif
 
+#if PHP_VERSION_ID >= 70200
+    #define im_ce_countable zend_ce_countable
+#else
+    #define im_ce_countable spl_ce_Countable
+#endif
 
 typedef struct _php_imagick_callback {
 	void ***thread_ctx;
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	zval user_callback;
 #else
 	zval *user_callback;
@@ -83,6 +100,11 @@ ZEND_BEGIN_MODULE_GLOBALS(imagick)
 	zend_bool locale_fix;
 	zend_bool progress_monitor;
 	zend_bool skip_version_check;
+
+	zend_bool set_single_thread;
+	zend_bool allow_zero_dimension_images;
+	im_long shutdown_sleep_count;
+
 	php_imagick_callback *progress_callback;
 #ifdef PHP_IMAGICK_ZEND_MM
 	MagickWand *keep_alive;
@@ -129,7 +151,7 @@ ZEND_EXTERN_MODULE_GLOBALS(imagick)
 #endif
 
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 
 /* Structure for Imagick object. */
 typedef struct _php_imagick_object  {
@@ -152,7 +174,7 @@ typedef struct _php_imagick_object  {
 #endif
 
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 
 /* Structure for ImagickDraw object. */
 typedef struct _php_imagickdraw_object  {
@@ -173,7 +195,7 @@ typedef struct _php_imagickdraw_object  {
 
 #endif
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 
 /* Structure for ImagickPixelIterator object. */
 typedef struct _php_imagickpixeliterator_object  {
@@ -207,7 +229,7 @@ typedef struct _php_imagickpixeliterator_object  {
 #endif
 
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 
 /* Structure for ImagickPixel object. */
 typedef struct _php_imagickpixel_object  {
@@ -231,7 +253,7 @@ typedef struct _php_imagickpixel_object  {
 
 
 #ifdef IMAGICK_WITH_KERNEL
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 /* Structure for ImagickKernel object. */
 typedef struct _php_imagickkernel_object  {
 	KernelInfo *kernel_info;
@@ -248,7 +270,7 @@ typedef struct _php_imagickkernel_object  {
 #endif
 
 //Object fetching.
-#ifdef ZEND_ENGINE_3 
+#if PHP_VERSION_ID >= 70000
 
 static inline php_imagick_object *php_imagick_fetch_object(zend_object *obj) {
 	return (php_imagick_object *)((char*)(obj) - XtOffsetOf(php_imagick_object, zo));
@@ -281,7 +303,7 @@ static inline php_imagickkernel_object *php_imagickkernel_fetch_object(zend_obje
 #endif
 
 // Object access
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	#define Z_IMAGICK_P(zv) php_imagick_fetch_object(Z_OBJ_P((zv)))
 	#define Z_IMAGICKDRAW_P(zv) php_imagickdraw_fetch_object(Z_OBJ_P((zv)))
 	#define Z_IMAGICKPIXEL_P(zv) php_imagickpixel_fetch_object(Z_OBJ_P((zv)))
@@ -298,7 +320,7 @@ static inline php_imagickkernel_object *php_imagickkernel_fetch_object(zend_obje
 #endif
 
 // String access
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	#define IM_ZVAL_STRING(zv, charstar) ZVAL_STRING(zv, charstar);
 	#define IM_RETURN_STRING(s) RETURN_STRING(s)
 #else
@@ -307,7 +329,7 @@ static inline php_imagickkernel_object *php_imagickkernel_fetch_object(zend_obje
 	#define IM_RETURN_STRING(s) RETURN_STRING(s, 0)
 #endif
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	#define IM_add_assoc_string(zv, key, charstr) add_assoc_string(zv, key, charstr)
 	#define IM_ZVAL_STRINGL(zv, charstr, length) ZVAL_STRINGL(zv, charstr, length)
 	#define IM_add_next_index_string(zv, charstr) add_next_index_string(zv, charstr)
@@ -321,7 +343,7 @@ static inline php_imagickkernel_object *php_imagickkernel_fetch_object(zend_obje
 	#define IM_LEN_TYPE int
 #endif
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	#define IM_ZEND_OBJECT zend_object
 #else 
 	#define IM_ZEND_OBJECT void
@@ -388,6 +410,12 @@ extern zend_class_entry *php_imagickkernel_exception_class_entry;
 
 /* Forward declarations (Imagick) */
 
+#if (MagickLibVersion >= 0x700 && MagickLibVersion >= 0x709)
+  #define PHP_IMAGICK_HAVE_HOUGHLINE 1
+#else
+  #define PHP_IMAGICK_HAVE_HOUGHLINE 0
+#endif
+
 /* The conditional methods */
 #if MagickLibVersion > 0x628
 PHP_METHOD(imagick, pingimageblob);
@@ -401,7 +429,9 @@ PHP_METHOD(imagick, compareimagelayers);
 PHP_METHOD(imagick, optimizeimagelayers);
 PHP_METHOD(imagick, uniqueimagecolors);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, getimagematte);
+#endif //#if MagickLibVersion < 0x700
 #endif
 PHP_METHOD(imagick, setimagematte);
 PHP_METHOD(imagick, adaptiveresizeimage);
@@ -418,13 +448,20 @@ PHP_METHOD(imagick, roundcornersimage);
 #endif
 PHP_METHOD(imagick, setiteratorindex);
 PHP_METHOD(imagick, getiteratorindex);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, transformimage);
+#endif // #if MagickLibVersion < 0x700
 #endif
 #if MagickLibVersion > 0x630
 #if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimageopacity);
 #endif
+#if MagickLibVersion >= 0x700
+PHP_METHOD(imagick, setimagealpha);
+#endif
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, orderedposterizeimage);
+#endif //#if MagickLibVersion < 0x700
 #endif
 #if MagickLibVersion > 0x631
 PHP_METHOD(imagick, polaroidimage);
@@ -432,7 +469,9 @@ PHP_METHOD(imagick, getimageproperty);
 PHP_METHOD(imagick, setimageproperty);
 PHP_METHOD(imagick, deleteimageproperty);
 PHP_METHOD(imagick, identifyformat);
+#if IM_HAVE_IMAGICK_SETIMAGEINTERPOLATEMETHOD
 PHP_METHOD(imagick, setimageinterpolatemethod);
+#endif
 PHP_METHOD(imagick, getimageinterpolatemethod);
 PHP_METHOD(imagick, linearstretchimage);
 PHP_METHOD(imagick, getimagelength);
@@ -501,6 +540,11 @@ PHP_METHOD(imagick, setimagegravity);
 #if MagickLibVersion > 0x645
 PHP_METHOD(imagick, importimagepixels);
 PHP_METHOD(imagick, deskewimage);
+
+
+#if PHP_IMAGICK_HAVE_HOUGHLINE
+PHP_METHOD(imagick, houghlineimage);
+#endif
 PHP_METHOD(imagick, segmentimage);
 PHP_METHOD(imagick, sparsecolorimage);
 PHP_METHOD(imagick, remapimage);
@@ -560,8 +604,10 @@ PHP_METHOD(imagick, nextimage);
 PHP_METHOD(imagick, haspreviousimage);
 PHP_METHOD(imagick, hasnextimage);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimageindex);
 PHP_METHOD(imagick, getimageindex);
+#endif // #if MagickLibVersion < 0x700
 #endif
 PHP_METHOD(imagick, setimageformat);
 PHP_METHOD(imagick, setimagefilename);
@@ -569,8 +615,10 @@ PHP_METHOD(imagick, getimagefilename);
 PHP_METHOD(imagick, getimageformat);
 PHP_METHOD(imagick, getimagemimetype);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, getimagesize);
 #endif
+#endif
 PHP_METHOD(imagick, getimagegeometry);
 PHP_METHOD(imagick, getimagecolors);
 PHP_METHOD(imagick, scaleimage);
@@ -616,7 +664,9 @@ PHP_METHOD(imagick, sampleimage);
 PHP_METHOD(imagick, solarizeimage);
 PHP_METHOD(imagick, shadowimage);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimageattribute);
+#endif // #if MagickLibVersion < 0x700
 #endif
 PHP_METHOD(imagick, setimagebackgroundcolor);
 #if MagickLibVersion >= 0x700
@@ -628,7 +678,9 @@ PHP_METHOD(imagick, setimagedelay);
 PHP_METHOD(imagick, setimagedepth);
 PHP_METHOD(imagick, setimagegamma);
 PHP_METHOD(imagick, setimageiterations);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimagemattecolor);
+#endif // #if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimagepage);
 PHP_METHOD(imagick, setimageprogressmonitor);
 #if MagickLibVersion > 0x653
@@ -672,7 +724,9 @@ PHP_METHOD(imagick, clipimage);
 PHP_METHOD(imagick, clippathimage);
 PHP_METHOD(imagick, coalesceimages);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, colorfloodfillimage);
+#endif //#if MagickLibVersion < 0x700
 #endif
 PHP_METHOD(imagick, colorizeimage);
 PHP_METHOD(imagick, compareimagechannels);
@@ -716,7 +770,9 @@ PHP_METHOD(imagick, getimagebordercolor);
 PHP_METHOD(imagick, getimagechanneldepth);
 PHP_METHOD(imagick, getimagechanneldistortion);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, getimagechannelextrema);
+#endif // #if MagickLibVersion < 0x700
 #endif
 PHP_METHOD(imagick, getimagechannelmean);
 PHP_METHOD(imagick, getimagechannelstatistics);
@@ -727,7 +783,9 @@ PHP_METHOD(imagick, getimagedelay);
 PHP_METHOD(imagick, getimagedepth);
 PHP_METHOD(imagick, getimagedistortion);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, getimageextrema);
+#endif // #if MagickLibVersion < 0x700
 #endif
 PHP_METHOD(imagick, getimagedispose);
 PHP_METHOD(imagick, getimagegamma);
@@ -736,9 +794,12 @@ PHP_METHOD(imagick, getimageheight);
 PHP_METHOD(imagick, getimagehistogram);
 PHP_METHOD(imagick, getimageinterlacescheme);
 PHP_METHOD(imagick, getimageiterations);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, getimagemattecolor);
+#endif //#if MagickLibVersion < 0x700 
 PHP_METHOD(imagick, getimagepage);
 PHP_METHOD(imagick, getimagepixelcolor);
+PHP_METHOD(imagick, setImagePixelColor);
 PHP_METHOD(imagick, getimageprofile);
 PHP_METHOD(imagick, getimageredprimary);
 PHP_METHOD(imagick, getimagerenderingintent);
@@ -761,16 +822,20 @@ PHP_METHOD(imagick, inversefouriertransformimage);
 PHP_METHOD(imagick, levelimage);
 PHP_METHOD(imagick, magnifyimage);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, mapimage);
 PHP_METHOD(imagick, mattefloodfillimage);
+#endif // #if MagickLibVersion < 0x700
 #endif
 #if MagickLibVersion < 0x700
 PHP_METHOD(imagick, medianfilterimage);
 #endif
 PHP_METHOD(imagick, negateimage);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, paintopaqueimage);
 PHP_METHOD(imagick, painttransparentimage);
+#endif //#if MagickLibVersion < 0x700
 #endif
 PHP_METHOD(imagick, previewimages);
 PHP_METHOD(imagick, profileimage);
@@ -857,6 +922,9 @@ PHP_METHOD(imagick, selectiveblurimage);
 #if MagickLibVersion >= 0x689
 PHP_METHOD(imagick, rotationalblurimage);
 #endif
+#if MagickLibVersion >= 0x687
+PHP_METHOD(imagick, optimizeimagetransparency);
+#endif
 #if MagickLibVersion >= 0x683
 PHP_METHOD(imagick, statisticimage);
 #endif
@@ -868,7 +936,9 @@ PHP_METHOD(imagick, setregistry);
 PHP_METHOD(imagick, getregistry);
 PHP_METHOD(imagick, listregistry);
 PHP_METHOD(imagick, morphology);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, filter);
+#endif // #if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setantialias);
 PHP_METHOD(imagick, getantialias);
 #if MagickLibVersion > 0x676
@@ -885,6 +955,9 @@ PHP_METHOD(imagick, compositeimagegravity);
 #if MagickLibVersion >= 0x692
 PHP_METHOD(imagick, localcontrastimage);
 #endif // #if MagickLibVersion >= 0x692
+#if MagickLibVersion >= 0x700
+PHP_METHOD(imagick, identifyimagetype);
+#endif // #if MagickLibVersion >= 0x700
 
 /* Forward declarations (ImagickDraw) */
 #if MagickLibVersion > 0x628
@@ -1092,6 +1165,7 @@ PHP_METHOD(imagickpixel, setcolorfrompixel);
 #endif //#if MagickLibVersion >= 0x693
 
 #ifdef IMAGICK_WITH_KERNEL
+PHP_METHOD(imagickkernel, __construct);
 PHP_METHOD(imagickkernel, frommatrix);
 PHP_METHOD(imagickkernel, frombuiltin);
 PHP_METHOD(imagickkernel, addkernel);
